import os

from txhttputil.site.FileUnderlayResource import FileUnderlayResource

from peek_platform import PeekPlatformConfig

mobileRoot = FileUnderlayResource()

def setupMobile():
    # Setup properties for serving the site
    mobileRoot.enableSinglePageApplication()

    # This dist dir is automatically generated, but check it's parent

    import peek_mobile
    frontendProjectDir = os.path.dirname(peek_mobile.__file__)
    distDir = os.path.join(frontendProjectDir, 'build-web', 'dist')

    distDirParent = os.path.dirname(distDir)
    if not os.path.isdir(distDirParent):
        raise NotADirectoryError(distDirParent)

    # Make the dist dir, otherwise addFileSystemRoot throws an exception.
    # It rebuilds at a later date
    os.makedirs(distDir, exist_ok=True)

    mobileRoot.addFileSystemRoot(distDir)


desktopRoot = FileUnderlayResource()

def setupDesktop():
    # Setup properties for serving the site
    desktopRoot.enableSinglePageApplication()

    # This dist dir is automatically generated, but check it's parent

    import peek_desktop
    frontendProjectDir = os.path.dirname(peek_desktop.__file__)
    distDir = os.path.join(frontendProjectDir, 'build-web', 'dist')

    distDirParent = os.path.dirname(distDir)
    if not os.path.isdir(distDirParent):
        raise NotADirectoryError(distDirParent)

    # Make the dist dir, otherwise addFileSystemRoot throws an exception.
    # It rebuilds at a later date
    os.makedirs(distDir, exist_ok=True)

    desktopRoot.addFileSystemRoot(distDir)



docSiteRoot = FileUnderlayResource()

def setupDocSite():
    # Setup properties for serving the site
    docSiteRoot.enableSinglePageApplication()

    # This dist dir is automatically generated, but check it's parent

    import peek_doc_user
    docProjectDir = os.path.dirname(peek_doc_user.__file__)
    distDir = os.path.join(docProjectDir, 'doc_dist')

    distDirParent = os.path.dirname(distDir)
    if not os.path.isdir(distDirParent):
        raise NotADirectoryError(distDirParent)

    # Make the dist dir, otherwise addFileSystemRoot throws an exception.
    # It rebuilds at a later date
    os.makedirs(distDir, exist_ok=True)

    docSiteRoot.addFileSystemRoot(distDir)



import logging

import os

from peek_platform.build_doc.DocBuilder import DocBuilder
from peek_platform.build_frontend.NativescriptBuilder import NativescriptBuilder
from peek_platform.build_frontend.WebBuilder import WebBuilder

logger = logging.getLogger(__name__)


class ClientFrontendBuildersMixin:

    def _buildMobile(self, loadedPlugins):
        # --------------------
        # Prepare the Peek Mobile

        from peek_platform import PeekPlatformConfig

        try:
            import peek_mobile
            mobileProjectDir = os.path.dirname(peek_mobile.__file__)

        except:
            logger.warning("Skipping builds of peek-mobile"
                           ", the package can not be imported")
            return

        nsBuilder = NativescriptBuilder(mobileProjectDir,
                                        "peek-mobile",
                                        PeekPlatformConfig.config,
                                        loadedPlugins)
        yield nsBuilder.build()

        mobileWebBuilder = WebBuilder(mobileProjectDir,
                                      "peek-mobile",
                                      PeekPlatformConfig.config,
                                      loadedPlugins)
        yield mobileWebBuilder.build()

    def _buildDesktop(self, loadedPlugins):
        # --------------------
        # Prepare the Peek Desktop
        from peek_platform import PeekPlatformConfig

        try:
            import peek_desktop
            desktopProjectDir = os.path.dirname(peek_desktop.__file__)

        except:
            logger.warning("Skipping builds of peek-desktop"
                           ", the package can not be imported")
            return

        desktopWebBuilder = WebBuilder(desktopProjectDir,
                                       "peek-desktop",
                                       PeekPlatformConfig.config,
                                       loadedPlugins)
        yield desktopWebBuilder.build()

    def _buildDocs(self, loadedPlugins):
        # --------------------
        # Prepare the User Docs
        from peek_platform import PeekPlatformConfig

        try:
            import peek_doc_user
            docProjectDir = os.path.dirname(peek_doc_user.__file__)

        except:
            logger.warning("Skipping builds of peek_doc_user"
                           ", the package can not be imported")
            return

        docBuilder = DocBuilder(docProjectDir,
                                "peek-doc-user",
                                PeekPlatformConfig.config,
                                loadedPlugins)
        yield docBuilder.build()



import logging
from typing import Type, Tuple, List

from twisted.internet.defer import inlineCallbacks

from peek_client.plugin.ClientFrontendBuildersMixin import ClientFrontendBuildersMixin
from peek_client.plugin.PeekClientPlatformHook import PeekClientPlatformHook
from peek_platform.plugin.PluginLoaderABC import PluginLoaderABC
from peek_plugin_base.PluginCommonEntryHookABC import PluginCommonEntryHookABC
from peek_plugin_base.client.PluginClientEntryHookABC import PluginClientEntryHookABC

logger = logging.getLogger(__name__)


class ClientPluginLoader(PluginLoaderABC, ClientFrontendBuildersMixin):
    _instance = None

    def __new__(cls, *args, **kwargs):
        assert cls._instance is None, "ClientPluginLoader is a singleton, don't construct it"
        cls._instance = PluginLoaderABC.__new__(cls)
        return cls._instance

    def __init__(self, *args, **kwargs):
        PluginLoaderABC.__init__(self, *args, **kwargs)

    @property
    def _entryHookFuncName(self) -> str:
        return "peekClientEntryHook"

    @property
    def _entryHookClassType(self):
        return PluginClientEntryHookABC

    @property
    def _platformServiceNames(self) -> List[str]:
        return ["client"]

    @inlineCallbacks
    def loadOptionalPlugins(self):
        yield PluginLoaderABC.loadOptionalPlugins(self)

        yield from self._buildMobile(self._loadedPlugins.values())

        yield from self._buildDesktop(self._loadedPlugins.values())

        yield from self._buildDocs(self._loadedPlugins.values())

    def unloadPlugin(self, pluginName: str):
        PluginLoaderABC.unloadPlugin(self, pluginName)

        # Remove the Plugin resource tree
        from peek_client.backend.SiteRootResource import mobileRoot
        try:
            mobileRoot.deleteChild(pluginName.encode())
        except KeyError:
            pass

        # Remove the Plugin resource tree
        from peek_client.backend.SiteRootResource import desktopRoot
        try:
            desktopRoot.deleteChild(pluginName.encode())
        except KeyError:
            pass

    @inlineCallbacks
    def _loadPluginThrows(self, pluginName: str,
                          EntryHookClass: Type[PluginCommonEntryHookABC],
                          pluginRootDir: str,
                          requiresService: Tuple[str, ...]) -> PluginCommonEntryHookABC:
        # Everyone gets their own instance of the plugin API
        platformApi = PeekClientPlatformHook(pluginName)

        pluginMain = EntryHookClass(pluginName=pluginName,
                                    pluginRootDir=pluginRootDir,
                                    platform=platformApi)

        # Load the plugin
        yield pluginMain.load()

        # Add all the resources required to serve the backend site
        # And all the plugin custom resources it may create
        from peek_client.backend.SiteRootResource import mobileRoot
        mobileRoot.putChild(pluginName.encode(), platformApi.rootMobileResource)

        from peek_client.backend.SiteRootResource import desktopRoot
        desktopRoot.putChild(pluginName.encode(), platformApi.rootDesktopResource)

        self._loadedPlugins[pluginName] = pluginMain



"""
 *
 *  Copyright Synerty Pty Ltd 2013
 *
 *  This software is proprietary, you are not free to copy
 *  or redistribute this code in any format.
 *
 *  All rights to this software are reserved by
 *  Synerty Pty Ltd
 *
 * Website : http://www.synerty.com
 * Support : support@synerty.com
 *
"""
import logging
import os

from jsoncfg.value_mappers import require_integer
from peek_platform.file_config.PeekFileConfigABC import PeekFileConfigABC
from peek_platform.file_config.PeekFileConfigDocBuildMixin import \
    PeekFileConfigDocBuildMixin
from peek_platform.file_config.PeekFileConfigFrontendDirMixin import \
    PeekFileConfigFrontendDirMixin
from peek_platform.file_config.PeekFileConfigOsMixin import PeekFileConfigOsMixin
from peek_platform.file_config.PeekFileConfigPeekServerClientMixin import \
    PeekFileConfigPeekServerClientMixin
from peek_platform.file_config.PeekFileConfigPlatformMixin import \
    PeekFileConfigPlatformMixin

logger = logging.getLogger(__name__)


class PeekClientConfig(PeekFileConfigABC,
                       PeekFileConfigPeekServerClientMixin,
                       PeekFileConfigPlatformMixin,
                       PeekFileConfigOsMixin,
                       PeekFileConfigFrontendDirMixin,
                       PeekFileConfigDocBuildMixin):
    """
    This class creates a basic client configuration
    """

    ### SERVER SECTION ###
    @property
    def mobileSitePort(self) -> int:
        with self._cfg as c:
            return c.server.mobileSitePort(8000, require_integer)

    @property
    def webSocketPort(self) -> int:
        with self._cfg as c:
            return c.server.webSocketPort(8001, require_integer)


    @property
    def desktopSitePort(self) -> int:
        with self._cfg as c:
            return c.server.desktopSitePort(8002, require_integer)


    @property
    def docSitePort(self) -> int:
        with self._cfg as c:
            return c.server.docSitePort(8005, require_integer)

